------------------------------------------------------------------------------
-- $Id: rx_cl_if.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- rx_cl_if.vhd            
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_cl_if.vhd
-- Version:         v3.00a
-- Description:     register block for ipic_to_temac
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DCW
-- History:
--  DCW      2004.05.07      -- First version
--
--  <initials>      <date> 
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.coregen_comp_defs.all;

-- synopsys translate_off
library XilinxCoreLib;
-- synopsys translate_on

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity rx_cl_if is
    generic (
             C_FAMILY        : string    := "virtex5";  
             C_RESET_ACTIVE  : std_logic := '1';
             C_TEMAC_TYPE    : integer   :=    0;  
               -- 0 - Virtex 5 hard TEMAC (FX, LXT, SXT devices)                
               -- 1 - Virtex 4 hard TEMAC (FX)               
               -- 2 - Soft TEMAC         
             C_CLIENT_DWIDTH : integer   :=  8
            );

    port    (
             takeRxStatsNow_p2     : out std_logic;
             llTakeRxStatsNow_d1   : in  std_logic;
             PauseLocalLinkFull    : in std_logic;
             FifoFullReset         : in std_logic;
             LLTemac_Clk           : in  std_logic;
             LLTemac_Rst           : in  std_logic;

             RXFIFO2IP_Almost_Full : in  std_logic;
             RXFIFO2IP_Full        : in  std_logic;
             Rx2Cl_IF_RdEn         : in  std_logic;
             Cl_IF2Rx_F_Mt         : out std_logic;
             Cl_IF2Rx_Data         : out std_logic_vector(0 to 15);
             Cl_IF2RxD_Val_Ind     : out std_logic_vector(0 to 1);
             Cl_IF2Rx_EndOfFrameMarker     : out std_logic;
             Cl_IF2Rx_FrameIsCorruptMarker : out std_logic;
             Cl_Fifo_Rd_Ack        : out std_logic;
             Rx2Cl_IF_Rd_Frm_Inds  : in  std_logic;
             Cl_If2Rx_End_Of_Pckt  : out std_logic;
             Rx_Cmplt              : in  std_logic;
             Cl_If2Rx_BadFrame     : out std_logic;
             Cl_If2Rx_GoodFrame    : out std_logic;
             Cl_If2Rx_FrameDrop    : out std_logic;

             Rx2Cl_IF_Rd_Frm_Stats : in  std_logic;
             Cl_If2Rx_Stats        : out std_logic_vector(26 downto 0);

             Rx_Cl_Clk             : in  std_logic;
             EmacClientRxBadFrame  : in  std_logic;
             EmacClientRxd         : in  std_logic_vector(7 downto 0);
             EmacClientRxdVld      : in  std_logic;
             EmacClientRxFrameDrop : in  std_logic;
             EmacClientRxGoodFrame : in  std_logic;
             EmacClientRxStats     : in  std_logic_vector(26 downto 0);
             EmacClientRxStatsVld  : in  std_logic;
             TakeRxStatsNow        : in  std_logic;
             EmacClientRxStats_i   : in  std_logic_vector(6 downto 0);
             SoftEmacClientRxStats : in  std_logic_vector(27 downto 0);
             Cl_If2Rx_StatsVld	   : out std_logic;
             
             ClFifoOverflow        : out std_logic
            );
end rx_cl_if;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture simulation of rx_cl_if is

------------------------------------------------------------------------------
--  Constant Declarations
------------------------------------------------------------------------------


--------------------------------------------------------------------------
-- Type Declarations
--------------------------------------------------------------------------

type RXCLSM_TYPE is (
                     PRE_IDLE,
                     IDLE,
                     BYTE_0,
                     BYTE_1,
                     W8_4_STATS,
                     W8_4_GOOD_BAD,
                     END_OF_PCKT
                    );

--------------------------------------------------------------------------
-- Signal Declarations
--------------------------------------------------------------------------
signal clFifoOverflow_i        : std_logic;

signal rxClSm_Cs : RXCLSM_TYPE;
signal rxClSm_Ns : RXCLSM_TYPE;

signal emacClientRxd_d         : std_logic_vector(7 downto 0);
signal emacClientRxdVld_d      : std_logic;
signal wr_en                   : std_logic;
signal fifo_full               : std_logic;
signal emacClientRxBadFrame_d  : std_logic;
signal cl_If2Rx_BadFrame_i     : std_logic;
signal emacClientRxGoodFrame_d : std_logic;
signal cl_If2Rx_GoodFrame_i    : std_logic;
signal emacClientRxFrameDrop_d : std_logic;
signal cl_If2Rx_FrameDrop_i    : std_logic;
signal eop                     : std_logic;
signal emacClientRxStatsVld_d  : std_logic;
signal cl_If2Rx_StatsVld_i     : std_logic;
signal cl_If2Rx_StatsVld_id1   : std_logic;
signal emacClientRxStats_d     : std_logic_vector(26 downto 0);
signal cl_If2Rx_Stats_i        : std_logic_vector(26 downto 0);

signal rxClIfSm_IDLE_i         : std_logic;
signal rx_Cmplt_d,rx_Cmplt_i   : std_logic;
signal cl_If2Rx_End_Of_Pckt_i  : std_logic;
signal eop_d2,eop_d3,eop_d4,eop_d1, eop_i : std_logic;
signal stretchClientRxStatsVld_i : std_logic;
signal stretchClientRxGoodFrame_i: std_logic;
signal stretchClientRxBadFrame_i : std_logic;

signal rxclsm_encoded            : std_logic_vector(0 to 2);

signal cl_IF2Rx_F_Mt_i           : std_logic;
signal cl_IF2Rx_Data_i           : std_logic_vector(0 to 15);
signal cl_IF2RxD_Val_Ind_i       : std_logic_vector(0 to 1);
signal cl_Fifo_Rd_Ack_i          : std_logic;

signal emacClientRxBadFrame_p1  : std_logic;
signal emacClientRxd_p1         : std_logic_vector(7 downto 0);
signal emacClientRxdVld_p1      : std_logic;
signal emacClientRxFrameDrop_p1 : std_logic;
signal emacClientRxGoodFrame_p1 : std_logic;
signal emacClientRxStats_p1     : std_logic_vector(26 downto 0);
signal emacClientRxStatsVld_p1  : std_logic;
signal takeRxStatsNow_p1        : std_logic;
signal emacClientRxStats_i_p1   : std_logic_vector(6 downto 0);
signal softEmacClientRxStats_p1 : std_logic_vector(27 downto 0);

signal endOfTheFrameMarker      : std_logic;
signal frameIsCorruptMarker     : std_logic;
signal almostFull               : std_logic;

signal cl_IF2Rx_EndOfFrameMarker_i     : std_logic;
signal cl_IF2Rx_FrameIsCorruptMarker_i : std_logic;

signal combo_rset : std_logic;
begin

  combo_rset <= LLTemac_Rst or FifoFullReset or PauseLocalLinkFull;
  --------------------------------------------------------------------------
  -- Concurrent signal assignments
  --------------------------------------------------------------------------

  Cl_IF2Rx_F_Mt      <= cl_IF2Rx_F_Mt_i;    
  Cl_IF2Rx_Data      <= cl_IF2Rx_Data_i;    
  Cl_IF2RxD_Val_Ind  <= cl_IF2RxD_Val_Ind_i;
  Cl_Fifo_Rd_Ack     <= cl_Fifo_Rd_Ack_i;   
  
  Cl_If2Rx_GoodFrame   <= cl_If2Rx_GoodFrame_i;
  Cl_If2Rx_BadFrame    <= cl_If2Rx_BadFrame_i;
  Cl_If2Rx_FrameDrop   <= cl_If2Rx_FrameDrop_i;
  Cl_If2Rx_Stats       <= cl_If2Rx_Stats_i;
  Cl_If2Rx_End_Of_Pckt <= cl_If2Rx_End_Of_Pckt_i;
  ClFifoOverflow       <= clFifoOverflow_i;

  Cl_IF2Rx_EndOfFrameMarker     <= cl_IF2Rx_EndOfFrameMarker_i;    
  Cl_IF2Rx_FrameIsCorruptMarker <= cl_IF2Rx_FrameIsCorruptMarker_i;

  --------------------------------------------------------------------------
  -- PIPE_EMACCLIENT_PROCESS: creates an early version of the inputs
  --------------------------------------------------------------------------
  
  PIPE_EMACCLIENT_PROCESS: process ( LLTemac_Rst, Rx_Cl_Clk )
  begin
      if (LLTemac_Rst = '1') then
          emacClientRxBadFrame_p1  <= '0';
          emacClientRxd_p1         <= (others => '0');
          emacClientRxdVld_p1      <= '0';
          emacClientRxFrameDrop_p1 <= '0';
          emacClientRxGoodFrame_p1 <= '0';
          emacClientRxStats_p1     <= (others => '0');
          emacClientRxStatsVld_p1  <= '0';
          takeRxStatsNow_p1        <= '0';
          takeRxStatsNow_p2        <= '0';
          emacClientRxStats_i_p1   <= (others => '0');
          softEmacClientRxStats_p1 <= (others => '0');
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          emacClientRxBadFrame_p1  <= EmacClientRxBadFrame;
          emacClientRxd_p1         <= EmacClientRxd;
          emacClientRxdVld_p1      <= EmacClientRxdVld;
          emacClientRxFrameDrop_p1 <= EmacClientRxFrameDrop;
          emacClientRxGoodFrame_p1 <= EmacClientRxGoodFrame;
          emacClientRxStats_p1     <= EmacClientRxStats;
          emacClientRxStatsVld_p1  <= EmacClientRxStatsVld;
          takeRxStatsNow_p1        <= TakeRxStatsNow;
          takeRxStatsNow_p2        <= takeRxStatsNow_p1;
          emacClientRxStats_i_p1   <= EmacClientRxStats_i;
          softEmacClientRxStats_p1 <= SoftEmacClientRxStats;
      end if;
  end process;
    
--  NOT_VIRTEX5: if(not(C_FAMILY = "virtex5")) generate
--  begin
--    --------------------------------------------------------------------------
--    -- Component Instantiation for Rx_Client_FIFO
--    --------------------------------------------------------------------------
--    I_RX_CLIENT_FIFO : fifo_generator_v3_2
--      generic map (
--        C_RD_FREQ                      => 100,
--        C_WR_RESPONSE_LATENCY          => 1, 
--        C_HAS_SRST                     => 0, 
--        C_HAS_RD_DATA_COUNT            => 1, 
--        C_DIN_WIDTH                    => 18, 
--        C_HAS_WR_DATA_COUNT            => 1, 
--        C_IMPLEMENTATION_TYPE          => 2, 
--        C_FAMILY                       => C_FAMILY, 
--        C_HAS_WR_RST                   => 0, 
--        C_WR_FREQ                      => 100,
--        C_UNDERFLOW_LOW                => 0, 
--        C_HAS_MEMINIT_FILE             => 0, 
--        C_HAS_OVERFLOW                 => 1, 
--        C_PRELOAD_LATENCY              => 1, 
--        C_DOUT_WIDTH                   => 18, 
--        C_RD_DEPTH                     => 16, 
--        C_DEFAULT_VALUE                => "BlankString", 
--        C_MIF_FILE_NAME                => "BlankString", 
--        C_HAS_UNDERFLOW                => 1, 
--        C_HAS_RD_RST                   => 0, 
--        C_HAS_ALMOST_FULL              => 1, 
--        C_HAS_RST                      => 1, 
--        C_DATA_COUNT_WIDTH             => 2,
--        C_HAS_WR_ACK                   => 1, 
--        C_WR_ACK_LOW                   => 0, 
--        C_COMMON_CLOCK                 => 0, 
--        C_RD_PNTR_WIDTH                => 5, 
--        C_HAS_ALMOST_EMPTY             => 1, 
--        C_RD_DATA_COUNT_WIDTH          => 2,
--        C_ENABLE_RLOCS                 => 0, 
--        C_WR_PNTR_WIDTH                => 5, 
--        C_OVERFLOW_LOW                 => 0, 
--        C_PROG_EMPTY_TYPE              => 0,
--        C_OPTIMIZATION_MODE            => 0, 
--        C_WR_DATA_COUNT_WIDTH          => 2,
--        C_PRELOAD_REGS                 => 0, 
--        C_DOUT_RST_VAL                 => "0", 
--        C_HAS_DATA_COUNT               => 0, 
--        C_PROG_FULL_THRESH_NEGATE_VAL  => 13,
--        C_WR_DEPTH                     => 16, 
--        C_PROG_EMPTY_THRESH_NEGATE_VAL => 3,
--        C_PROG_EMPTY_THRESH_ASSERT_VAL => 2,
--        C_HAS_VALID                    => 1, 
--        C_INIT_WR_PNTR_VAL             => 0, 
--        C_PROG_FULL_THRESH_ASSERT_VAL  => 14,
--        C_USE_FIFO16_FLAGS             => 0, 
--        C_HAS_BACKUP                   => 0, 
--        C_VALID_LOW                    => 0, 
--        C_PRIM_FIFO_TYPE               => "2kx18",
--        C_COUNT_TYPE                   => 0, 
--        C_PROG_FULL_TYPE               => 0, 
--        C_MEMORY_TYPE                  => 1 
--        )				    
--      port map (
--        Din(7  downto 0)          => EmacClientRxd,             -- in
--        Din(15 downto 8)          => emacClientRxd_d,           -- in
--        Din(16)                   => EmacClientRxdVld,          -- in
--        Din(17)                   => emacClientRxdVld_d,        -- in
--        Rd_clk                    => LLTemac_Clk,               -- in
--        Rd_en                     => Rx2Cl_IF_RdEn,             -- in
--        RST                       => LLTemac_Rst,               -- in
--        Wr_clk                    => Rx_Cl_Clk,                 -- in
--        Wr_en                     => wr_en,                     -- in 
--        ALMOST_EMPTY              => open,                      -- out 
--        ALMOST_FULL               => open,                      -- out 
--        Dout(15 downto  0)        => Cl_IF2Rx_Data(0 to 15),    -- out
--        Dout(17 downto 16)        => Cl_IF2RxD_Val_Ind(0 to 1), -- out
--        Empty                     => Cl_IF2Rx_F_Mt,             -- out
--        Full                      => fifo_full,                 -- out
--        OVERFLOW                  => open,                      -- out 
--        VALID                     => Cl_Fifo_Rd_Ack,            -- out
--        RD_DATA_COUNT             => open,                      -- out ;
--        UNDERFLOW                 => open,                      -- out 
--        WR_ACK                    => open,                      -- out 
--        WR_DATA_COUNT             => open,                      -- out 
--        CLK                       => open,                      -- in  
--        BACKUP                    => open,                      -- in  
--        BACKUP_MARKER             => open,                      -- in  
--        PROG_EMPTY_THRESH         => open,                      -- in  
--        PROG_EMPTY_THRESH_ASSERT  => open,                      -- in  
--        PROG_EMPTY_THRESH_NEGATE  => open,                      -- in  
--        PROG_FULL_THRESH          => open,                      -- in  
--        PROG_FULL_THRESH_ASSERT   => open,                      -- in  
--        PROG_FULL_THRESH_NEGATE   => open,                      -- in  
--        RD_RST                    => open,                      -- in  
--        SRST                      => open,                      -- in  
--        WR_RST                    => open,                      -- in  
--        DATA_COUNT                => open,                      -- out 
--        PROG_EMPTY                => open,                      -- out 
--        PROG_FULL                 => open                       -- out 
--        );
--  end generate NOT_VIRTEX5;
--
--  IS_VIRTEX5: if(C_FAMILY = "virtex5") generate
--  begin
--    --------------------------------------------------------------------------
--    -- Component Instantiation for Rx_Client_FIFO
--    --------------------------------------------------------------------------
--    I_RX_CLIENT_FIFO : fifo_generator_v3_2
--      generic map (
--        C_RD_FREQ                      => 150,
--        C_WR_RESPONSE_LATENCY          => 1, 
--        C_HAS_SRST                     => 0, 
--        C_HAS_RD_DATA_COUNT            => 0, 
--        C_DIN_WIDTH                    => 18, 
--        C_HAS_WR_DATA_COUNT            => 0, 
--        C_IMPLEMENTATION_TYPE          => 4, 
--        C_FAMILY                       => C_FAMILY, 
--        C_HAS_WR_RST                   => 0, 
--        C_WR_FREQ                      => 150,
--        C_UNDERFLOW_LOW                => 0, 
--        C_HAS_MEMINIT_FILE             => 0, 
--        C_HAS_OVERFLOW                 => 0, 
--        C_PRELOAD_LATENCY              => 1, 
--        C_DOUT_WIDTH                   => 18, 
--        C_RD_DEPTH                     => 16, 
--        C_DEFAULT_VALUE                => "BlankString", 
--        C_MIF_FILE_NAME                => "BlankString", 
--        C_HAS_UNDERFLOW                => 0, 
--        C_HAS_RD_RST                   => 0, 
--        C_HAS_ALMOST_FULL              => 0, 
--        C_HAS_RST                      => 1, 
--        C_DATA_COUNT_WIDTH             => 4,
--        C_HAS_WR_ACK                   => 0, 
--        C_WR_ACK_LOW                   => 0, 
--        C_COMMON_CLOCK                 => 0, 
--        C_RD_PNTR_WIDTH                => 4, 
--        C_HAS_ALMOST_EMPTY             => 0, 
--        C_RD_DATA_COUNT_WIDTH          => 4,
--        C_ENABLE_RLOCS                 => 0, 
--        C_WR_PNTR_WIDTH                => 4, 
--        C_OVERFLOW_LOW                 => 0, 
--        C_PROG_EMPTY_TYPE              => 0,
--        C_OPTIMIZATION_MODE            => 0, 
--        C_WR_DATA_COUNT_WIDTH          => 4,
--        C_PRELOAD_REGS                 => 0, 
--        C_DOUT_RST_VAL                 => "0", 
--        C_HAS_DATA_COUNT               => 0, 
--        C_PROG_FULL_THRESH_NEGATE_VAL  => 1020,
--        C_WR_DEPTH                     => 16, 
--        C_PROG_EMPTY_THRESH_NEGATE_VAL => 6,
--        C_PROG_EMPTY_THRESH_ASSERT_VAL => 5,
--        C_HAS_VALID                    => 1, 
--        C_INIT_WR_PNTR_VAL             => 0, 
--        C_PROG_FULL_THRESH_ASSERT_VAL  => 1021,
--        C_USE_FIFO16_FLAGS             => 0, 
--        C_HAS_BACKUP                   => 0, 
--        C_VALID_LOW                    => 0, 
--        C_PRIM_FIFO_TYPE               => "1kx18",
--        C_COUNT_TYPE                   => 0, 
--        C_PROG_FULL_TYPE               => 0, 
--        C_MEMORY_TYPE                  => 4 
--        )				    
--      port map (
--        Din(7  downto 0)          => EmacClientRxd,             -- in
--        Din(15 downto 8)          => emacClientRxd_d,           -- in
--        Din(16)                   => EmacClientRxdVld,          -- in
--        Din(17)                   => emacClientRxdVld_d,        -- in
--        Rd_clk                    => LLTemac_Clk,               -- in
--        Rd_en                     => Rx2Cl_IF_RdEn,             -- in
--        RST                       => LLTemac_Rst,               -- in
--        Wr_clk                    => Rx_Cl_Clk,                 -- in
--        Wr_en                     => wr_en,                     -- in 
--        ALMOST_EMPTY              => open,                      -- out 
--        ALMOST_FULL               => open,                      -- out 
--        Dout(15 downto  0)        => Cl_IF2Rx_Data(0 to 15),    -- out
--        Dout(17 downto 16)        => Cl_IF2RxD_Val_Ind(0 to 1), -- out
--        Empty                     => Cl_IF2Rx_F_Mt,             -- out
--        Full                      => fifo_full,                 -- out
--        OVERFLOW                  => open,                      -- out 
--        VALID                     => Cl_Fifo_Rd_Ack,            -- out
--        RD_DATA_COUNT             => open,                      -- out ;
--        UNDERFLOW                 => open,                      -- out 
--        WR_ACK                    => open,                      -- out 
--        WR_DATA_COUNT             => open,                      -- out 
--        CLK                       => open,                      -- in  
--        BACKUP                    => open,                      -- in  
--        BACKUP_MARKER             => open,                      -- in  
--        PROG_EMPTY_THRESH         => open,                      -- in  
--        PROG_EMPTY_THRESH_ASSERT  => open,                      -- in  
--        PROG_EMPTY_THRESH_NEGATE  => open,                      -- in  
--        PROG_FULL_THRESH          => open,                      -- in  
--        PROG_FULL_THRESH_ASSERT   => open,                      -- in  
--        PROG_FULL_THRESH_NEGATE   => open,                      -- in  
--        RD_RST                    => open,                      -- in  
--        SRST                      => open,                      -- in  
--        WR_RST                    => open,                      -- in  
--        DATA_COUNT                => open,                      -- out 
--        PROG_EMPTY                => open,                      -- out 
--        PROG_FULL                 => open                       -- out 
--        );
--
--  end generate IS_VIRTEX5;
--                     
  I_RX_CLIENT_FIFO : async_fifo_v6_1
      generic map (
                   C_DATA_WIDTH       => 20,
                   C_ENABLE_RLOCS     => 0,
                   C_FIFO_DEPTH       => 32,
                   C_HAS_ALMOST_EMPTY => 0,
                   C_HAS_ALMOST_FULL  => 1,
                   C_HAS_RD_ACK       => 1,
                   C_HAS_RD_COUNT     => 0,
                   C_HAS_RD_ERR       => 0,
                   C_HAS_WR_ACK       => 0,
                   C_HAS_WR_COUNT     => 0,
                   C_HAS_WR_ERR       => 0,
                   C_RD_ACK_LOW       => 0,
                   C_RD_COUNT_WIDTH   => 2,
                   C_RD_ERR_LOW       => 0,
                   C_USE_BLOCKMEM     => 1,
                   C_WR_ACK_LOW       => 0,
                   C_WR_COUNT_WIDTH   => 2,
                   C_WR_ERR_LOW       => 0
                  )

      port map    (
                   Din(7  downto 0)   => EmacClientRxd_p1,             -- in
                   Din(15 downto 8)   => emacClientRxd_d,           -- in
                   Din(16)            => EmacClientRxdVld_p1,          -- in
                   Din(17)            => emacClientRxdVld_d,        -- in
                   Din(18)            => endOfTheFrameMarker,       -- in
                   Din(19)            => frameIsCorruptMarker,      -- in
                   Wr_en              => wr_en,                     -- in 
                   Wr_clk             => Rx_Cl_Clk,                 -- in
                   Rd_en              => Rx2Cl_IF_RdEn,             -- in
                   Rd_clk             => LLTemac_Clk,               -- in
                   Ainit              => combo_rset, --LLTemac_Rst,               -- in
                   Dout(15 downto  0) => cl_IF2Rx_Data_i(0 to 15),  -- out
                   Dout(17 downto 16) => cl_IF2RxD_Val_Ind_i(0 to 1), -- out
                   Dout(18)           => cl_IF2Rx_EndOfFrameMarker_i, -- out
                   Dout(19)           => cl_IF2Rx_FrameIsCorruptMarker_i, -- out
                   Full               => fifo_full,                 -- out
                   Empty              => cl_IF2Rx_F_Mt_i,           -- out
                   Almost_full        => almostFull,                -- out
                   Almost_empty       => open,                      -- out
                   Wr_count           => open,                      -- out
                   Rd_count           => open,                      -- out
                   Rd_ack             => cl_Fifo_Rd_Ack_i,          -- out
                   Rd_err             => open,                      -- out
                   Wr_ack             => open,                      -- out
                   Wr_err             => open                       -- out
                  );

  --------------------------------------------------------------------------
  -- LTCH_FIFO_OVRFLW_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  LTCH_FIFO_OVRFLW_PROCESS : process ( LLTemac_Rst, Rx_Cl_Clk )
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          clFifoOverflow_i <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          clFifoOverflow_i <= (fifo_full and wr_en) or   -- set   - write while FIFO is full
                             (clFifoOverflow_i);         -- hold  - until reset
      end if;
  end process;

  --------------------------------------------------------------------------
  -- BYTE_0_REG_PROCESS
  --------------------------------------------------------------------------
  
  BYTE_0_REG_PROCESS: process ( LLTemac_Rst, Rx_Cl_Clk )
  begin
      if (LLTemac_Rst = '1') then
          emacClientRxdVld_d <= '0';
          emacClientRxd_d    <= (others => '0');
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          emacClientRxdVld_d <= EmacClientRxdVld_p1;
          emacClientRxd_d    <= EmacClientRxd_p1;
      end if;
  end process;


  --------------------------------------------------------------------------
  -- EMAC_ENDOFPACKET_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  EMAC_ENDOFPACKET_PROCESS : process ( combo_rset, Rx_Cl_Clk )
  begin
      if (combo_rset = C_RESET_ACTIVE) then
          eop_d1 <= '0';
          eop_d2 <= '0';
          eop_d3 <= '0';
          eop_d4 <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          eop_d1 <= eop or                         -- set   - indicator from state machine below
                    (eop_d1 and                    -- hold  - until captured by LLTemac_Clk
                    not cl_If2Rx_End_Of_Pckt_i);   -- clear - captured by LLTemac_Clk
          eop_d2 <= eop_d1;
          eop_d3 <= eop_d2;
          eop_d4 <= eop_d3;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- RX_ENDOFPACKET_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_ENDOFPACKET_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE) then
              cl_If2Rx_End_Of_Pckt_i <= '0';
          else
              cl_If2Rx_End_Of_Pckt_i <= eop_d4;              -- set   - from Rx_Cl_Clk hold circuit above
 
          end if;
      end if;
  end process;


  --------------------------------------------------------------------------
  -- EMAC_BADFRAME_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  EMAC_BADFRAME_PROCESS : process ( combo_rset, Rx_Cl_Clk )
  begin
      if (combo_rset = C_RESET_ACTIVE) then
          emacClientRxBadFrame_d <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          emacClientRxBadFrame_d <= EmacClientRxBadFrame_p1 or -- set   - indicator from hard temac core
                     (emacClientRxBadFrame_d and            -- hold  - until captured by LLTemac_Clk
                      not cl_If2Rx_BadFrame_i);             -- clear - captured by LLTemac_Clk
      end if;
  end process;

  --------------------------------------------------------------------------
  -- RX_BADFRAME_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_BADFRAME_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE) then
              cl_If2Rx_BadFrame_i <= '0';
          else
              cl_If2Rx_BadFrame_i <= emacClientRxBadFrame_d or -- set   - from Rx_Cl_Clk hold circuit above
                       (cl_If2Rx_BadFrame_i and                -- hold  - until the packet is finished
                        not (llTakeRxStatsNow_d1 or RXFIFO2IP_Almost_Full or RXFIFO2IP_Full));             -- clear - packet finished indicator from rx_sm
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- EMAC_GOODFRAME_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  EMAC_GOODFRAME_PROCESS : process ( combo_rset, Rx_Cl_Clk )
  begin
      if (combo_rset = C_RESET_ACTIVE) then
          emacClientRxGoodFrame_d <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          emacClientRxGoodFrame_d <= EmacClientRxGoodFrame_p1 or -- set   - indicator from hard temac core
                     (emacClientRxGoodFrame_d and             -- hold  - until captured by LLTemac_Clk
                      not cl_If2Rx_GoodFrame_i);              -- clear - captured by LLTemac_Clk
      end if;
  end process;

  --------------------------------------------------------------------------
  -- RX_GOODFRAME_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_GOODFRAME_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE) then
              cl_If2Rx_GoodFrame_i <= '0';
          else
              cl_If2Rx_GoodFrame_i <= emacClientRxGoodFrame_d or -- set   - from Rx_Cl_Clk hold circuit above
                       (cl_If2Rx_GoodFrame_i and                 -- hold  - until the packet is finished
                        not (llTakeRxStatsNow_d1 or RXFIFO2IP_Almost_Full or RXFIFO2IP_Full));               -- clear - packet finished indicator from rx_sm
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- STCH_GOODFRAME_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  STCH_GOODFRAME_PROCESS : process (combo_rset, Rx_Cl_Clk)
  begin
      if (combo_rset = C_RESET_ACTIVE) then
        stretchClientRxGoodFrame_i <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
        stretchClientRxGoodFrame_i <= EmacClientRxGoodFrame_p1 or -- set   - indicator from hard temac core
                               (stretchClientRxGoodFrame_i and -- hold  - until the packet is finished
                               not (eop or fifo_full));                       -- clear - packet finished indicator from rx_clsm
      end if;
  end process;

  --------------------------------------------------------------------------
  -- STCH_BADFRAME_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  STCH_BADFRAME_PROCESS : process (combo_rset, Rx_Cl_Clk)
  begin
      if (combo_rset = C_RESET_ACTIVE) then
        stretchClientRxBadFrame_i <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
        stretchClientRxBadFrame_i <= EmacClientRxBadFrame_p1 or -- set   - indicator from hard temac core
                              (stretchClientRxBadFrame_i and -- hold  - until the packet is finished
                              not (eop or fifo_full));                      -- clear - packet finished indicator from rx_clsm
      end if;
  end process;

  --------------------------------------------------------------------------
  -- STCH_RXSTATSVLD_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  STCH_RXSTATSVLD_PROCESS : process (combo_rset, Rx_Cl_Clk)
  begin
      if (combo_rset = C_RESET_ACTIVE) then
        stretchClientRxStatsVld_i <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
        stretchClientRxStatsVld_i <= EmacClientRxStatsVld_p1 or -- set   - indicator from hard temac core
                              (stretchClientRxStatsVld_i and -- hold  - until the packet is finished
                              not (eop or fifo_full));                      -- clear - packet finished indicator from rx_clsm
      end if;
  end process;

  --------------------------------------------------------------------------
  -- EMAC_FRAMEDROP_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  EMAC_FRAMEDROP_PROCESS : process ( combo_rset, Rx_Cl_Clk )
  begin
      if (combo_rset = C_RESET_ACTIVE) then
          emacClientRxFrameDrop_d <= '0';
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          emacClientRxFrameDrop_d <= EmacClientRxFrameDrop_p1 or -- set   - indicator from hard temac core
                     (emacClientRxFrameDrop_d and             -- hold  - until captured by LLTemac_Clk
                      not cl_If2Rx_FrameDrop_i);              -- clear - captured by LLTemac_Clk
      end if;
  end process;

  --------------------------------------------------------------------------
  -- RX_FRAMEDROP_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_FRAMEDROP_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE) then
              cl_If2Rx_FrameDrop_i <= '0';
          else
              cl_If2Rx_FrameDrop_i <= emacClientRxFrameDrop_d or -- set   - from Rx_Cl_Clk hold circuit above
                     (cl_If2Rx_FrameDrop_i and                   -- hold  - until the packet is finished
                      not (llTakeRxStatsNow_d1 or RXFIFO2IP_Almost_Full or RXFIFO2IP_Full));                 -- clear - packet finished indicator from rx_sm
          end if;
      end if;
  end process;

  GEN_V4V5HARD : if(not(C_TEMAC_TYPE = 2)) generate
    --------------------------------------------------------------------------
    -- EMAC_RXSTATS_PROCESS - Rx_Cl_Clk based V4 and V5 hard TEMACs
    --------------------------------------------------------------------------

    EMAC_RXSTATS_PROCESS : process ( combo_rset, Rx_Cl_Clk )
    begin
        if (combo_rset = C_RESET_ACTIVE or eop_d4 ='1') then
            emacClientRxStatsVld_d <= '0';
            emacClientRxStats_d    <= (others => '0');
        elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
            emacClientRxStatsVld_d <= EmacClientRxStatsVld_p1 or -- set   - indicator from hard temac core
                    (emacClientRxStatsVld_d and               -- hold  - until captured by LLTemac_Clk
                     not cl_If2Rx_StatsVld_i);                -- clear - captured by LLTemac_Clk
            if (TakeRxStatsNow_p1 = '1') then
                for i in 0 to 26 loop
                    emacClientRxStats_d(i) <= EmacClientRxStats_p1(i);  -- set when valid
                end loop;
            else
                for i in 0 to 26 loop
                    emacClientRxStats_d(i) <= emacClientRxStats_d(i);-- hold until valid again
                end loop;
            end if;
        end if;
    end process;
  end generate;
  GEN_SOFT : if(C_TEMAC_TYPE = 2) generate
    --------------------------------------------------------------------------
    -- EMAC_RXSTATS_PROCESS - Rx_Cl_Clk based soft TEMAC
    --------------------------------------------------------------------------

    EMAC_RXSTATS_PROCESS : process ( combo_rset, Rx_Cl_Clk )
    begin
        if (combo_rset = C_RESET_ACTIVE) then
            emacClientRxStatsVld_d <= '0';
            emacClientRxStats_d    <= (others => '0');
        elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
            emacClientRxStatsVld_d <= EmacClientRxStatsVld_p1 or -- set   - indicator from hard temac core
                    (emacClientRxStatsVld_d and               -- hold  - until captured by LLTemac_Clk
                     not cl_If2Rx_StatsVld_i);                -- clear - captured by LLTemac_Clk
            if (EmacClientRxStatsVld_p1 = '1') then
                for i in 0 to 26 loop
                    emacClientRxStats_d(i) <= SoftEmacClientRxStats_p1(i);  -- set when valid
                end loop;
            else
                for i in 0 to 26 loop
                    emacClientRxStats_d(i) <= emacClientRxStats_d(i);-- hold until valid again
                end loop;
            end if;
        end if;
    end process;
  end generate;

  --------------------------------------------------------------------------
  -- RX_RXSTATS_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_RXSTATS_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE or llTakeRxStatsNow_d1 ='1') then
              cl_If2Rx_StatsVld_i   <= '0';
              cl_If2Rx_StatsVld_id1 <= '0';
              cl_If2Rx_Stats_i      <= (others => '0');
          else
              cl_If2Rx_StatsVld_id1 <= cl_If2Rx_StatsVld_i;
              cl_If2Rx_StatsVld_i <= emacClientRxStatsVld_d or -- set   - from Rx_Cl_Clk hold circuit above
                       (cl_If2Rx_StatsVld_i and                -- hold  - until the packet is finished
                        not Rx2Cl_IF_Rd_Frm_Stats);            -- clear - packet finished indicator from rx_sm
              for i in 0 to 26 loop
                  cl_If2Rx_Stats_i(i) <= emacClientRxStats_d(i); -- set   - from Rx_Cl_Clk circuit above
              end loop;
          end if;
      end if;
  end process;
  Cl_If2Rx_StatsVld <= not (cl_If2Rx_StatsVld_i) and cl_If2Rx_StatsVld_id1;
  --------------------------------------------------------------------------
  -- Rx Client State Machine
  -- RXCLSM_REGS_PROCESS: registered process of the state machine
  -- RXCLSM_CMB_PROCESS:  combinatorial next-state logic
  --------------------------------------------------------------------------
  
  RXCLSM_REGS_PROCESS: process ( combo_rset, Rx_Cl_Clk )
  begin
      if (combo_rset = '1') then
          rxClSm_Cs <= PRE_IDLE;
      elsif (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          rxClSm_Cs <= rxClSm_Ns;
      end if;
  end process;
  
  RXCLSM_CMB_PROCESS: process (
                               RxClSm_Cs,
                               rx_Cmplt_i,
                               EmacClientRxBadFrame_p1,
                               EmacClientRxd_p1,
                               EmacClientRxdVld,
                               EmacClientRxdVld_p1,
                               stretchClientRxGoodFrame_i,
                               stretchClientRxBadFrame_i,
                               cl_If2Rx_StatsVld_i,
                               stretchClientRxStatsVld_i,
                               fifo_full,
                               cl_If2Rx_BadFrame_i,
                               cl_If2Rx_GoodFrame_i,
                               cl_If2Rx_FrameDrop_i,
                               Rx_Cmplt,
                               almostFull
                              )

  begin
  
    frameIsCorruptMarker <= '0';
    endOfTheFrameMarker  <= not(EmacClientRxdVld); --early version of signal for normal operation i.e. no corruption because of full fifo
    
    case rxClSm_Cs is
      when PRE_IDLE =>
          rxclsm_encoded <= "110";
          rxClIfSm_IDLE_i <= '0';          
          wr_en           <= '0';
          eop             <= '0';
          if ((EmacClientRxdVld_p1 = '0') and (almostFull = '0') and (fifo_full = '0')) then -- make sure there's enough room to at least save a corrupt packet flag entry
              rxClSm_Ns <= IDLE;
          else
              rxClSm_Ns <= PRE_IDLE; -- Wait until the end of the current frame to guarantee starting at the beginning of a frame
          end if;
          
      when IDLE =>
          rxclsm_encoded <= "000";
          rxClIfSm_IDLE_i           <= '1';          
          wr_en <= '0';
          eop   <= '0';
          if (EmacClientRxdVld_p1 = '1') then
              rxClSm_Ns <= BYTE_0;
          else
              rxClSm_Ns <= IDLE;
          end if;

      when BYTE_0 =>
          rxclsm_encoded  <= "001";
          rxClIfSm_IDLE_i <= '0';          
          wr_en           <= '1';
          eop             <= '0';
          if (almostFull = '0') then
            rxClSm_Ns     <= BYTE_1;
          else -- overflowed the client FIFO corrupting the frame
            frameIsCorruptMarker <= '1';
            endOfTheFrameMarker  <= '1';
            rxClSm_Ns            <= PRE_IDLE;
          end if;

      when BYTE_1 =>
          rxclsm_encoded <= "010";
          rxClIfSm_IDLE_i           <= '0';          
          wr_en <= '0';
          eop   <= '0';
          if (EmacClientRxdVld_p1 = '1') then
              rxClSm_Ns <= BYTE_0;
          else
              rxClSm_Ns <= W8_4_STATS;
          end if;

      when W8_4_STATS =>
          rxclsm_encoded <= "011";
          rxClIfSm_IDLE_i           <= '0';          
          wr_en <= '0';
          eop   <= '0';
          if (stretchClientRxStatsVld_i = '1') then
              rxClSm_Ns <= W8_4_GOOD_BAD;
          elsif ( (stretchClientRxGoodFrame_i = '1') or
                  (stretchClientRxBadFrame_i  = '1') )then
              rxClSm_Ns <= W8_4_GOOD_BAD;
          else
              rxClSm_Ns <= W8_4_STATS;
          end if;

      when W8_4_GOOD_BAD =>
          rxclsm_encoded <= "100";
          rxClIfSm_IDLE_i           <= '0';          
          wr_en <= '0';
          eop   <= '0';
          if (stretchClientRxStatsVld_i = '1') and
           ( (stretchClientRxGoodFrame_i = '1') or
             (stretchClientRxBadFrame_i  = '1') ) then
              rxClSm_Ns <= END_OF_PCKT;
          else
              rxClSm_Ns <= W8_4_GOOD_BAD;
          end if;

      when END_OF_PCKT =>
          rxclsm_encoded <= "101";
          rxClIfSm_IDLE_i           <= '0';          
          wr_en     <= '0';
          eop       <= '1';
          if (rx_Cmplt_i = '1') then
              rxClSm_Ns <= PRE_IDLE;
          else
              rxClSm_Ns <= END_OF_PCKT;
          end if;

      when others   =>  -- default to IDLE
          rxclsm_encoded <= "110";
          rxClIfSm_IDLE_i           <= '1';          
          wr_en <= '0';
          eop   <= '1';
          rxClSm_Ns <= PRE_IDLE;
    end case;
  end process;

  --------------------------------------------------------------------------
  -- RX_CMPLT_PROCESS - LLTemac_Clk based
  --------------------------------------------------------------------------

  RX_CMPLT_PROCESS : process ( combo_rset, LLTemac_Clk )
  begin
      if (combo_rset = C_RESET_ACTIVE) then
          rx_Cmplt_d <= '0';
      elsif (LLTemac_Clk'event and LLTemac_Clk = '1') then
          rx_Cmplt_d <= Rx_Cmplt or                 -- set   - indicator from hard temac core
                        (rx_Cmplt_d and             -- hold  - until captured by LLTemac_Clk
                        not rx_Cmplt_i);            -- clear - captured by Rx_Cl_Clk
      end if;
  end process;

  --------------------------------------------------------------------------
  -- RXCMPLT_PROCESS - Rx_Cl_Clk based
  --------------------------------------------------------------------------

  RXCMPLT_PROCESS : process (Rx_Cl_Clk)
  begin
      if (Rx_Cl_Clk'event and Rx_Cl_Clk = '1') then
          if (combo_rset = C_RESET_ACTIVE) then
              rx_Cmplt_i <= '0';
          else
              rx_Cmplt_i <= rx_Cmplt_d or               -- set   - from LLTemac_Clk hold circuit above
                            (rx_Cmplt_i and             -- hold  - until the packet is finished
                            not (rxClIfSm_IDLE_i) );    -- clear - packet finished indicator from rx_sm
          end if;
      end if;
  end process;

end simulation;
