library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity rx_csum_top is
  port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    err      : in  std_logic;
    eop      : in  std_logic;
    cscntrl  : in  std_logic;
    data_in  : in  std_logic_vector(0 to 31);
    release  : out std_logic;
    wr_ack   : in  std_logic;
    wr_req   : out std_logic;
    wr_data  : out std_logic_vector(0 to 15)
    );
end entity;

architecture beh of rx_csum_top is


  signal checksum_a : std_logic_vector(16 downto 0);
  signal checksum_b : std_logic_vector(16 downto 0);
  signal checksum   : std_logic_vector(15 downto 0);
  signal enable_a   : std_logic;
  signal enable_b   : std_logic;
  signal word_cnt   : integer range 0 to 3;
  signal eop_i  : std_logic;
  signal eop_2i : std_logic;
  signal eop_3i : std_logic;
  signal eop_4i : std_logic;

  begin


    ---------------------------------------------------------------------------
    -- This process enables the checksum after the 14 byte ethernet header
    ---------------------------------------------------------------------------
    process(clk)
      begin
        if(rising_edge(clk)) then
          if(rst='1' or eop='1' or eop_4i='1' or err='1') then
            enable_a<='0';
            enable_b<='0';
            word_cnt<= 0;
          else
            if(wr_ack='1') then
              if(word_cnt=2) then
                enable_b <= '1';
                enable_a <= enable_b;
              else
                word_cnt <= word_cnt + 1;
                enable_a <= '0';
                enable_b <= '0';
              end if;
            end if;
          end if;
        end if;
    end process;


    ---------------------------------------------------------------------------
    -- this is where the checksum is calculated
    ---------------------------------------------------------------------------
    wr_data <= --(others => '0') when cscntrl='0' else
               not checksum when conv_integer(checksum)=0 else
               checksum;

    process(clk)
      begin
        if(rising_edge(clk)) then
          if(rst='1' or err='1' or eop_4i='1') then
            checksum_a   <= (others => '0');
          else
            if(eop='1') then
              checksum_a <= '0'&checksum_a(15 downto 0) + checksum_a(16);
            elsif(enable_a='1' and wr_ack='1') then
              checksum_a <= '0'&checksum_a(15 downto 0) + data_in(0 to 15) + checksum_a(16);
            end if;
          end if;
        end if;
    end process;


    process(clk)
      begin
        if(rising_edge(clk)) then
          if(rst='1' or err='1' or eop_4i='1') then
            eop_4i  <= '0';
            eop_3i  <= '0';
            eop_2i  <= '0';
            eop_i   <= '0';
            checksum_b  <= (others => '0');
          else
            eop_i   <= eop;
            eop_2i  <= eop_i;
            eop_3i  <= eop_2i;
            eop_4i  <= eop_3i;
            if(eop='1' or eop_2i='1') then
              checksum_b <= '0'&checksum_b(15 downto 0) + checksum_b(16);
            elsif(eop_i='1') then
              checksum_b <= '0'&checksum_b(15 downto 0) + checksum_a(15 downto 0);
            elsif(enable_b='1' and wr_ack='1') then
              checksum_b <= '0'&checksum_b(15 downto 0) + checksum_b(16) + data_in(16 to 31);
            end if;
          end if;
        end if;
   end process;

   wr_req <= eop_3i;
   release <= eop_4i;
   checksum <= checksum_b(15 downto 0);

end beh;


                      
