------------------------------------------------------------------------------
-- $Id: rx_footer.vhd,v 1.1 2008/02/08 04:15:10 shurt Exp $
------------------------------------------------------------------------------
-- rx_sm.vhd            
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_footer.vhd
-- Version:         v1.00a
-- Description:     Attaches the footer to the end of packet
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DCW      2006.05.16      -- First version
--
--  <initials>      <date> 
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity rx_footer is
    generic(
      C_RX_LEN_LENGTH   : integer := 14
      );
    port(
      LLTemac_Clk           : in  std_logic;
      LLTemac_Rst           : in  std_logic;                 
      Footer_Done           : out std_logic;                  
      Rx_Csum_Data          : in  std_logic_vector(0 to 15);
      Rx_Pckt_Length        : in  std_logic_vector(0 to C_RX_LEN_LENGTH-1);
      Good_Bad_n            : in  std_logic_vector(0 to 1);
      RXFIFO2LL_Data        : in  std_logic_vector(0 to 31);
      TemacLL_SOP_n         : in  std_logic;
      TemacLL_EOP_n         : in  std_logic;
      TemacLL_SRC_RDY_n     : in  std_logic;
      TemacLL_DST_RDY_n     : in  std_logic;      
      TemacLL_Data          : out std_logic_vector(0 to 31)
      );
            
end rx_footer;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture beh of rx_footer is

                 
                   
------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal eop_i               : std_logic;
signal eop_2i              : std_logic;
signal eop_3i              : std_logic;
signal eop_4i              : std_logic;
signal eop_5i              : std_logic;
signal eop_6i              : std_logic;
signal eop_7i              : std_logic;
signal eop_8i              : std_logic;
signal tied_low            : std_logic_vector(0 to 31) := (others=>'0');
signal rx_pckt_valid       : std_logic;
signal rx_pckt_valid_i     : std_logic;

begin


------------------------------------------------------------------------------
-- Concurrent Assignments
------------------------------------------------------------------------------

Footer_Done <= eop_8i;

------------------------------------------------------------------------------
-- Pipeline EOF 8 clocks
------------------------------------------------------------------------------

PIPE_EOF : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1') then
            eop_i  <= '0';
            eop_2i <= '0';
            eop_3i <= '0';
            eop_4i <= '0';
            eop_5i <= '0';
            eop_6i <= '0';
            eop_7i <= '0';
            eop_8i <= '0';
         elsif(TemacLL_DST_RDY_n='0')then
            eop_i  <= not TemacLL_EOP_n;
            eop_2i <= eop_i;
            eop_3i <= eop_2i;
            eop_4i <= eop_3i;
            eop_5i <= eop_4i;
            eop_6i <= eop_5i;
            eop_7i <= eop_6i;
            eop_8i <= eop_7i;
         end if;
      end if;
   end process;
            


------------------------------------------------------------------------------
--  This process makes the valid packet signal used by Attach Footer process
------------------------------------------------------------------------------
      
PACKET_VALID_PROC : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1' or TemacLL_EOP_n='0') then
            rx_pckt_valid_i <= '0';
         elsif(TemacLL_SOP_n='0') then
            rx_pckt_valid_i <= '1';
         end if;
      end if;
   end process;

rx_pckt_valid <= rx_pckt_valid_i or not TemacLL_SOP_n;

------------------------------------------------------------------------------
-- Attach footer
------------------------------------------------------------------------------


ATTACH_FOOTER : process(LLTemac_Rst, TemacLL_SRC_RDY_n, eop_7i, eop_8i, 
                        Rx_Csum_Data, Rx_Pckt_Length, TemacLL_DST_RDY_n, 
                        tied_low, RXFIFO2LL_Data, Rx_Pckt_Valid)
   begin
      if(LLTemac_Rst='1') then
         TemacLL_Data <= (others=>'0');
      elsif((TemacLL_SRC_RDY_n='0') and 
            (TemacLL_DST_RDY_n='0') and
            (eop_7i='1')) then
            TemacLL_Data <= tied_low(0 to 15)&Rx_Csum_Data;            
      elsif((TemacLL_SRC_RDY_n='0') and 
            (TemacLL_DST_RDY_n='0') and
            (eop_8i='1')) then
            TemacLL_Data <= tied_low(0 to 17)&Rx_Pckt_Length; 
      elsif(Rx_Pckt_Valid='1') then
         TemacLL_Data <= RXFIFO2LL_Data;
      else
         TemacLL_Data <= (others=>'0');
      end if;
   end process;


end beh;
