------------------------------------------------------------------------------
-- $Id: rx_ll_if.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- rx_ll_if.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_ll_if.vhd
-- Version:         v1.00a
-- Description:     Receive interface between the receive and status fifos and
--                  LL.  Data is formatted into the local link prototcol
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DRP      2006.05.17      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Libraries used;
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


library xps_ll_temac_v1_01_b;
use xps_ll_temac_v1_01_b.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

entity rx_ll_if is
    generic (
             C_RESET_ACTIVE  : std_logic := '1';
             C_BUS_DWIDTH    : integer   := 32;
             C_CLIENT_DWIDTH : integer   :=  8;
             C_RX_LEN_LENGTH : integer   := 14
            );

    port    (
             LocalLinkPaused          : out std_logic;
             PauseLocalLinkFull       : in  std_logic;
             FifoFullReset            : in  std_logic;
             ResetFifosOutOfSync      : out std_logic;
             LLTemac_Clk              : in  std_logic;
             LLTemac_Rst              : in  std_logic;
             
             RSFIFO2LL_Empty          : in  std_logic;             
             RSFIFO2LL_Und            : in  std_logic;
             RSFIFO2LL_Rdack          : in  std_logic;             
             LL2RSFIFO_Rden           : out std_logic;
             RSFIFO2LL_Data           : in  std_logic_vector(0 to 35);

             RXFIFO2LL_Empty          : in  std_logic;
             RXFIFO2LL_Und            : in  std_logic;
             RXFIFO2LL_Rdack          : in  std_logic;
             LL2RXFIFO_Rden           : out std_logic;             
             RXFIFO2LL_Data           : in  std_logic_vector(0 to 35);
             
             TemacLL_SOF_n            : out std_logic;
             TemacLL_SOP_n            : out std_logic;
             TemacLL_EOF_n            : out std_logic;
             TemacLL_EOP_n            : out std_logic;
             TemacLL_SRC_RDY_n        : out std_logic;
             TemacLL_DST_RDY_n        : in  std_logic;
             TemacLL_REM              : out std_logic_vector(0 to 3);
             TemacLL_Data             : out std_logic_vector(0 to 31)
            );
end rx_ll_if;

------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------

architecture beh of rx_ll_if is

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal footer_done            : std_logic;
signal dump_bad_pckt          : std_logic;
signal temacLL_EOP_n_i        : std_logic;
signal temacLL_SOP_n_i        : std_logic;
signal temacLL_EOF_n_i        : std_logic;
signal temacLL_SRC_RDY_n_i    : std_logic;
signal rx_rem_sel             : std_logic_vector(0 to 1);
--signal rx_pckt_valid          : std_logic;
--signal rx_pckt_valid_i        : std_logic;
signal rx_pckt_length         : std_logic_vector(0 to 13);
signal remainder              : integer range 0 to 3;
signal rem_enc                : std_logic_vector(0 to 2*C_CLIENT_DWIDTH/8-1);
signal rem_enc_i              : integer range 0 to 3;
signal rx_eop                 : std_logic;
signal rxFIFO2LL_Eop          : std_logic;

signal lL2RSFIFO_Rden_i       : std_logic;
signal lL2RXFIFO_Rden_i       : std_logic;
signal temacLL_SOF_n_i        : std_logic;
signal temacLL_REM_i          : std_logic_vector(0 to 3);
signal temacLL_Data_i         : std_logic_vector(0 to 31);
signal sM_encoded             : std_logic_vector(0 to 2);

begin

  LL2RSFIFO_Rden    <= lL2RSFIFO_Rden_i;
  LL2RXFIFO_Rden    <= lL2RXFIFO_Rden_i;

  TemacLL_SOF_n     <= temacLL_SOF_n_i;    
  TemacLL_REM       <= temacLL_REM_i(0 to 3);      
  TemacLL_Data      <= temacLL_Data_i(0 to 31);     
  
------------------------------------------------------------------------------
-- Concurrent Signal Assignments
------------------------------------------------------------------------------

TemacLL_EOP_n     <= temacLL_EOP_n_i;
TemacLL_SOP_n     <= temacLL_SOP_n_i;
TemacLL_EOF_n     <= temacLL_EOF_n_i;
TemacLL_SRC_RDY_n <= temacLL_SRC_RDY_n_i;
rx_pckt_length    <= RSFIFO2LL_Data(16 to 29);
rem_enc           <= RXFIFO2LL_Data(34 to 35);

------------------------------------------------------------------------------
--  Component Instantiations
------------------------------------------------------------------------------

I_RX_LL_IF_SM : entity xps_ll_temac_v1_01_b.rx_ll_if_sm
   generic map(
      C_RESET_ACTIVE          => C_RESET_ACTIVE
      )
    port map(
      LocalLinkPaused         => LocalLinkPaused,   	      -- out
      PauseLocalLinkFull      => PauseLocalLinkFull,	      -- in 
      FifoFullReset           => FifoFullReset,     	      -- in 
      ResetFifosOutOfSync     => ResetFifosOutOfSync,         -- out
      SM_encoded              => sM_encoded,                  -- out
      LLTemac_Clk             => LLTemac_Clk,                 -- in  
      LLTemac_Rst             => LLTemac_Rst,                 -- in           
      Footer_Done             => footer_done,                 -- in  
      Dump_Bad_Pckt           => dump_bad_pckt,               -- out          
      RXFIFO2LL_Empty         => RXFIFO2LL_empty,             -- in  
      RXFIFO2LL_Und           => '0',                         -- in  
      RXFIFO2LL_Eop           => RXFIFO2LL_Data(33),--rx_eop,--rxFIFO2LL_Eop,              -- in
      LL2RXFIFO_Rden          => lL2RXFIFO_Rden_i,              -- out
      RSFIFO2LL_Empty         => RSFIFO2LL_Empty,             -- in  
      RSFIFO2LL_Und           => '0',                         -- in  
      RSFIFO2LL_Pckt_Status   => RSFIFO2LL_Data(30 to 35),    -- in  
      LL2RSFIFO_Rden          => lL2RSFIFO_Rden_i,              -- out          
      TemacLL_DST_RDY_n       => TemacLL_DST_RDY_n,           -- in  
      TemacLL_SOF_n           => temacLL_SOF_n_i,               -- out 
      TemacLL_SOP_n           => temacLL_SOP_n_i,             -- out 
      TemacLL_EOF_n           => temacLL_EOF_n_i,             -- out 
      TemacLL_EOP_n           => temacLL_EOP_n_i,             -- out 
      TemacLL_SRC_RDY_n       => temacLL_SRC_RDY_n_i          -- out 
      );   
                                                                                   
I_RX_FOOTER : entity xps_ll_temac_v1_01_b.rx_footer 
   generic map(
      C_RX_LEN_LENGTH         => C_RX_LEN_LENGTH
      )
   port map(
      LLTemac_Clk             => LLTemac_Clk,                 -- in  
      LLTemac_Rst             => LLTemac_Rst,                 -- in  
      Footer_Done             => footer_done,                 -- out  
      Rx_Csum_Data            => RSFIFO2LL_Data(0 to 15),     -- in  
      Rx_Pckt_Length          => RSFIFO2LL_Data(16 to 29),    -- in  
      Good_Bad_n              => RSFIFO2LL_Data(30 to 31),    -- in  
      RXFIFO2LL_Data          => RXFIFO2LL_Data(0 to 31),     -- in
      TemacLL_SOP_n           => temacLL_SOP_n_i,             -- in
      TemacLL_EOP_n           => temacLL_EOP_n_i,             -- in  
      TemacLL_SRC_RDY_n       => temacLL_SRC_RDY_n_i,         -- in  
      TemacLL_DST_RDY_n       => TemacLL_DST_RDY_n,           -- in  
      TemacLL_Data            => temacLL_Data_i                 -- out 
      );


------------------------------------------------------------------------------
--  Receieve remainder decoder decodes 2-bit remainder to a 4-bit remainder
------------------------------------------------------------------------------

rem_enc_i  <= to_integer(unsigned(rem_enc));

process(LLTemac_Rst, rem_enc_i, TemacLL_DST_RDY_n, temacLL_SRC_RDY_n_i,
        temacLL_EOP_n_i)
   begin
      if(LLTemac_Rst='1') then
         temacLL_REM_i <= "1111";
         
      elsif(TemacLL_DST_RDY_n='0' and temacLL_SRC_RDY_n_i='0') then
         if(temacLL_EOP_n_i='0') then
            case rem_enc_i is                        
               when 1 => 
                  temacLL_REM_i <= "0111";
            
               when 2 => 
                  temacLL_REM_i <= "0011";
               
               when 3 => 
                  temacLL_REM_i <= "0001";
               
               when others => 
                  temacLL_REM_i <= "0000";
            
            end case;         
         else
            temacLL_REM_i <= "0000";
         end if;      
      else
         temacLL_REM_i <= "1111";      
      end if;
   end process;
   
------------------------------------------------------------------------------
-- Rising Edge detector process for RXFIFO2LL_Eop used by the state machine
------------------------------------------------------------------------------ 

--RX_EOP_DETECT : process(LLTemac_Clk)
--   begin
--      if(rising_edge(LLTemac_Clk)) then
--         if(LLTemac_Rst='1') then
--            rx_eop <= '0';
--         else
--            rx_eop <= RXFIFO2LL_Data(33);
--         end if;
--      end if;
--   end process;

--rxFIFO2LL_Eop <= RXFIFO2LL_Data(33) and not rx_eop;
      
      
process(LLTemac_Rst, RXFIFO2LL_Data, temacLL_EOP_n_i,LLTemac_Clk)
   begin
   if(rising_edge(LLTemac_Clk)) then
      if(LLTemac_Rst='1' or temacLL_EOP_n_i='0') then
         rx_eop <= '0';
      elsif(RXFIFO2LL_Data(33)='1') then
         rx_eop <= '1';
      end if;
   end if;
   end process;
     
end beh;
