------------------------------------------------------------------------------
-- $Id: tx_cl_if.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- tx_cl_if.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        tx_cl_if.vhd
-- Version:         v3.00a
-- Description:     register block for ipic_to_temac
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DCW
-- History:
--  DCW      2004.05.07      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.coregen_comp_defs.all;

-- synopsys translate_off
library XilinxCoreLib;
-- synopsys translate_on

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity tx_cl_if is
    generic (
             C_RESET_ACTIVE  : std_logic := '1';
             C_CLIENT_DWIDTH : integer   :=  8
            );

    port    (
             LLTemac_Clk            : in  std_logic;
             LLTemac_Rst            : in  std_logic;

             Cl_Fifo_WrEn           : in  std_logic;
             TxD2Cl_fifo            : in  std_logic_vector(0 to 17);
             Cl_Fifo_MT_n           : out std_logic;
             Cl_Fifo_full           : out std_logic;

             Tx2ClientUnderRun      : in  std_logic;
             Tx2ClientPauseReq      : in  std_logic;
             ClientEmacPauseReq     : out std_logic;
             Client2TxCollision     : out std_logic;
             Client2TxRetransmit    : out std_logic;

             Tx_Cl_Clk              : in  std_logic;
             ClientEmacTxd          : out std_logic_vector(7 downto 0);
             ClientEmacTxdVld       : out std_logic;
             ClientEmacTxdVldMsw    : out std_logic;
             ClientEmacTxFirstByte  : out std_logic;
             ClientEmacTxUnderRun   : out std_logic;
             EmacClientTxAck        : in  std_logic;
             EmacClientTxCollision  : in  std_logic;
             EmacClientTxRetransmit : in  std_logic
            );
end tx_cl_if;

architecture simulation of tx_cl_if is

--------------------------------------------------------------------------
-- Type Declarations
--------------------------------------------------------------------------

type TXCLSM_TYPE is (
                     IDLE,
                     RD_FIRST,
                     WAIT_ACK,
                     FIRST_BYT2,
                     RD_NXT,
                     NXT_BYT2
                    );

--------------------------------------------------------------------------
-- Signal Declarations
--------------------------------------------------------------------------
signal cl_Fifo_MT_n_int           : std_logic;
signal cl_Fifo_full_int           : std_logic;
signal clientEmacTxd_int          : std_logic_vector(7 downto 0);
signal clientEmacTxdVld_int       : std_logic;
signal clientEmacTxFirstByte_int  : std_logic;

signal txClSm_Cs : TXCLSM_TYPE;
signal txClSm_Ns : TXCLSM_TYPE;

signal byte_0   : std_logic_vector(7  downto 0);
signal byte_1   : std_logic_vector(7  downto 0);
signal byte_0V  : std_logic;
signal byte_1V  : std_logic;
signal rd_en    : std_logic;
signal ainit    : std_logic;
signal empty    : std_logic;
signal wr_cnt   : std_logic_vector(3 downto 0);

signal cl_tx_start          : std_logic;
signal cl_tx_start_sync     : std_logic;

signal client2TxColl_i      : std_logic;
signal client2TxRetran_i    : std_logic;
signal clientEmacTxUnd_i    : std_logic;
signal clientEmacPauseReq_i : std_logic;

signal emacClientTxColl_d   : std_logic;
signal emacClientTxRetran_d : std_logic;
signal tx2ClientUnd_d       : std_logic;
signal tx2ClientPauseReq_d  : std_logic;

signal fifo_empty_i : std_logic;

signal txclsm_encoded            : std_logic_vector(0 to 2);


begin


  --------------------------------------------------------------------------
  -- Concurrent signal assignments
  --------------------------------------------------------------------------
  ClientEmacPauseReq    <= clientEmacPauseReq_i;   
  Client2TxCollision    <= client2TxColl_i;   
  Client2TxRetransmit   <= client2TxRetran_i;  
  
  ClientEmacTxd         <= clientEmacTxd_int(7 downto 0);        
  ClientEmacTxdVld      <= clientEmacTxdVld_int;     
  ClientEmacTxdVldMsw   <= '0';  -- not used for 8 bit Client  
  ClientEmacTxFirstByte <= clientEmacTxFirstByte_int;
  ClientEmacTxUnderRun  <= clientEmacTxUnd_i; 

  Cl_Fifo_MT_n          <= cl_Fifo_MT_n_int;         
  cl_Fifo_MT_n_int      <= not fifo_empty_i;
  Cl_Fifo_full          <= cl_Fifo_full_int;         
  cl_Fifo_full_int      <= (wr_cnt(3) and wr_cnt(2)) or (wr_cnt(3) and wr_cnt(1) and wr_cnt(0));
  cl_tx_start           <= wr_cnt(2) or wr_cnt(3);

  pipeIt1: FDR
    port map (
      Q => fifo_empty_i,  
      C => LLTemac_Clk,         
      D => empty,
      R => LLTemac_Rst          
    );

  pipeIt2: FDR
    port map (
      Q => cl_tx_start_sync,  
      C => Tx_Cl_Clk,         
      D => cl_tx_start,
      R => LLTemac_Rst          
    );

  --------------------------------------------------------------------------
  -- RESET_PROCESS
  --------------------------------------------------------------------------
  
  RESET_PROCESS  : process (LLTemac_Rst)
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          ainit <= '1';
      else
          ainit <= '0';
      end if;
  end process;

  --------------------------------------------------------------------------
  -- Component Instantiations
  --------------------------------------------------------------------------

  I_TX_CLIENT_FIFO : async_fifo_v6_1
      generic map (
                   C_DATA_WIDTH       => 18,
                   C_ENABLE_RLOCS     => 0,
                   C_FIFO_DEPTH       => 15,
                   C_HAS_ALMOST_EMPTY => 0,
                   C_HAS_ALMOST_FULL  => 0,
                   C_HAS_RD_ACK       => 0,
                   C_HAS_RD_COUNT     => 0,
                   C_HAS_RD_ERR       => 0,
                   C_HAS_WR_ACK       => 0,
                   C_HAS_WR_COUNT     => 1,
                   C_HAS_WR_ERR       => 0,
                   C_RD_ACK_LOW       => 0,
                   C_RD_COUNT_WIDTH   => 2,
                   C_RD_ERR_LOW       => 0,
                   C_USE_BLOCKMEM     => 1,
                   C_WR_ACK_LOW       => 0,
                   C_WR_COUNT_WIDTH   => 4,
                   C_WR_ERR_LOW       => 0   
                  )

      port map    (
                   Din               => TxD2Cl_fifo,   -- in
                   Wr_en             => Cl_Fifo_WrEn,  -- in 
                   Wr_clk            => LLTemac_Clk,    -- in
                   Rd_en             => rd_en,         -- in
                   Rd_clk            => Tx_Cl_Clk,     -- in
                   Ainit             => ainit,         -- in
                   Dout(7  downto 0) => byte_0,        -- out
                   Dout(15 downto 8) => byte_1,        -- out
                   Dout(16)          => byte_0V,       -- out
                   Dout(17)          => byte_1V,       -- out
                   Full              => open,          -- out
                   Empty             => empty,         -- out
                   Almost_full       => open,          -- out
                   Almost_empty      => open,          -- out
                   Wr_count          => wr_cnt,        -- out
                   Rd_count          => open,          -- out
                   Rd_ack            => open,          -- out
                   Rd_err            => open,          -- out
                   Wr_ack            => open,          -- out
                   Wr_err            => open           -- out
                  );

  --------------------------------------------------------------------------
  -- EMAC_COLL_PROCESS
  --------------------------------------------------------------------------

  EMAC_COLL_PROCESS : process ( LLTemac_Rst, Tx_Cl_Clk )
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          emacClientTxColl_d <= '0';
      elsif (Tx_Cl_Clk'event and Tx_Cl_Clk = '1') then
          emacClientTxColl_d <= EmacClientTxCollision or 
                               (emacClientTxColl_d and not client2TxColl_i);
      end if;
  end process;

  --------------------------------------------------------------------------
  -- TX_COLL_PROCESS
  --------------------------------------------------------------------------

  TX_COLL_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (LLTemac_Rst = C_RESET_ACTIVE) then
              client2TxColl_i <= '0';
          else
              client2TxColl_i <= emacClientTxColl_d;
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- EMAC_RETRAN_PROCESS
  --------------------------------------------------------------------------

  EMAC_RETRAN_PROCESS : process ( LLTemac_Rst, Tx_Cl_Clk )
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          emacClientTxRetran_d <= '0';
      elsif (Tx_Cl_Clk'event and Tx_Cl_Clk = '1') then
          emacClientTxRetran_d <= EmacClientTxRetransmit or 
                               (emacClientTxRetran_d and not client2TxRetran_i);
      end if;
  end process;

  --------------------------------------------------------------------------
  -- TX_RETRAN_PROCESS
  --------------------------------------------------------------------------

  TX_RETRAN_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (LLTemac_Rst = C_RESET_ACTIVE) then
              client2TxRetran_i <= '0';
          else
              client2TxRetran_i <= emacClientTxRetran_d;
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- TX_PAUSE_REQUEST_PROCESS
  --------------------------------------------------------------------------

  TX_PAUSE_REQUEST_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (LLTemac_Rst = C_RESET_ACTIVE) then
              tx2ClientPauseReq_d <= '0';
          else
              tx2ClientPauseReq_d <= Tx2ClientPauseReq or   -- set   - indicator from TPP reg
                                (tx2ClientPauseReq_d and    -- hold  - until captured by Tx_Cl_Clk
                                not clientEmacPauseReq_i);  -- clear - captured by Tx_Cl_Clk
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- EMAC_PAUSE_REQUEST_PROCESS
  --------------------------------------------------------------------------

  EMAC_PAUSE_REQUEST_PROCESS : process ( LLTemac_Rst, Tx_Cl_Clk )
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          clientEmacPauseReq_i <= '0';
      elsif (Tx_Cl_Clk'event and Tx_Cl_Clk = '1') then
          clientEmacPauseReq_i <= tx2ClientPauseReq_d;  -- set   - from LLTemac_Clk hold circuit above
      end if;
  end process;

  --------------------------------------------------------------------------
  -- TX_UNDERRUN_PROCESS
  --------------------------------------------------------------------------

  TX_UNDERRUN_PROCESS : process (LLTemac_Clk)
  begin
      if (LLTemac_Clk'event and LLTemac_Clk = '1') then
          if (LLTemac_Rst = C_RESET_ACTIVE) then
              tx2ClientUnd_d <= '0';
          else
              tx2ClientUnd_d <= Tx2ClientUnderRun or
                                (tx2ClientUnd_d and not clientEmacTxUnd_i);
          end if;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- EMAC_UNDERRUN_PROCESS
  --------------------------------------------------------------------------

  EMAC_UNDERRUN_PROCESS : process ( LLTemac_Rst, Tx_Cl_Clk )
  begin
      if (LLTemac_Rst = C_RESET_ACTIVE) then
          clientEmacTxUnd_i <= '0';
      elsif (Tx_Cl_Clk'event and Tx_Cl_Clk = '1') then
          clientEmacTxUnd_i <= tx2ClientUnd_d;
      end if;
  end process;

  --------------------------------------------------------------------------
  -- Tx Client State Machine
  -- TXCLSM_REGS_PROCESS: registered process of the state machine
  -- TXCLSM_CMB_PROCESS:  combinatorial next-state logic
  --------------------------------------------------------------------------
  
  TXCLSM_REGS_PROCESS: process ( LLTemac_Rst, Tx_Cl_Clk )
  begin
      if (LLTemac_Rst = '1') then
          txClSm_Cs <= IDLE;
      elsif (Tx_Cl_Clk'event and Tx_Cl_Clk = '1') then
          txClSm_Cs <= txClSm_Ns;
      end if;
  end process;
  
  TXCLSM_CMB_PROCESS: process (
                               txClSm_Cs,
                               cl_tx_start_sync,
                               EmacClientTxAck,
                               byte_0,
                               byte_0V,
                               byte_1,
                               byte_1V,
                               empty
                              )
  begin

      case txClSm_Cs is

              when IDLE =>
                  txclsm_encoded <= "000";
                  rd_en                 <= '0';
                  clientEmacTxd_int         <= byte_0;
                  clientEmacTxdVld_int      <= '0';
                  clientEmacTxFirstByte_int <= '0';
                  if (cl_tx_start_sync = '1') then
                      txClSm_Ns <= RD_FIRST;
                  else
                      txClSm_Ns <= IDLE;
                  end if;

              when RD_FIRST =>
                  txclsm_encoded <= "001";
                  rd_en                 <= '1';
                  clientEmacTxd_int         <= byte_0;
                  clientEmacTxdVld_int      <= '0';
                  clientEmacTxFirstByte_int <= '0';
                  txClSm_Ns             <= WAIT_ACK;

              when WAIT_ACK =>
                  txclsm_encoded <= "010";
                  clientEmacTxd_int         <= byte_1;
                  clientEmacTxdVld_int      <= byte_1V;
                  clientEmacTxFirstByte_int <= '0';
                  if (EmacClientTxAck = '1') then
                      rd_en     <= '0';
                      txClSm_Ns <= FIRST_BYT2;
                  elsif (byte_1V = '0') then
                      rd_en     <= '0'; -- new to remove latch
                    txClSm_Ns <= RD_FIRST;                    
                  else
                      rd_en     <= '0';
                      txClSm_Ns <= WAIT_ACK;
                  end if;

              when FIRST_BYT2 =>
                  txclsm_encoded <= "011";
                  rd_en                 <= '1';
                  clientEmacTxd_int         <= byte_0;
                  clientEmacTxdVld_int      <= byte_0V;
                  clientEmacTxFirstByte_int <= '1';
                  txClSm_Ns             <= RD_NXT;

              when RD_NXT =>
                  txclsm_encoded <= "100";
                  rd_en                 <= '0';
                  clientEmacTxd_int         <= byte_1;
                  clientEmacTxdVld_int      <= byte_1V;
                  clientEmacTxFirstByte_int <= '0';
                  txClSm_Ns <= NXT_BYT2;

              when NXT_BYT2 =>
                  txclsm_encoded <= "101";
                  rd_en                 <= '1';
                  clientEmacTxd_int         <= byte_0;
                  clientEmacTxdVld_int      <= byte_0V;
                  clientEmacTxFirstByte_int <= '0';
                  if (empty = '1')then
                      txClSm_Ns <= IDLE;
                  else
                      txClSm_Ns <= RD_NXT;
                  end if;

      when others   =>  -- default to IDLE
                  txclsm_encoded <= "000";
                  rd_en                 <= '0';
                  clientEmacTxd_int         <= byte_0;
                  clientEmacTxdVld_int      <= '0';
                  clientEmacTxFirstByte_int <= '0';
                  txClSm_Ns <= IDLE;

      end case;
  end process;

end simulation;
